/// <reference types="vite/client" />

import { AxiosInstance } from 'axios'
import { ComponentConfig } from './CrudEl/core'

declare module '*.vue' {
    import type { DefineComponent } from 'vue'
    const component: DefineComponent<{}, {}, any>
    export default component
}

/** CrudEl类型命名空间 */
declare namespace CrudElType {
    interface Res {
        code: number
        message: string
    }
    interface ResData<T = Record<string, any>> extends Res {
        /** 数据对象 */
        data: T[]
        /** 分页信息 */
        page: Record<string, any>
        additional: any
    }

    interface PropsType {
        /** 当前页面的BaseUrl */
        url?: string
        /** 数据源配置 */
        source?: SourceOptType
        /** 单主键 */
        rowKey?: string
        /** 多主键 */
        rowKeys?: string[]
        /** 快捷查询name */
        keyName?: string

        /** 复写函数处理调用 */
        hooks?: Partial<CrudFunction>

        debug?: boolean
        axiosService?: AxiosInstance
        isAfterMountQuery?: boolean
        FormStore?: ComponentConfig['FormStore']
        TableStore?: ComponentConfig['TableStore']
        PageStore?: ComponentConfig['PageStore']
        PermissionNames?: ComponentConfig['PermissionNames']
        Permissions?: ComponentConfig['Permissions']
        ApiRouteNames?: ComponentConfig['ApiRouteNames']
        ApiNames?: ComponentConfig['ApiNames']
    }

    interface SourceType {
        /** 搜索表单参数 并 排序 */
        filter: string[]
        /** 编辑表单参数 并 排序 */
        editor: string[]
        /** 表格列参数 并 排序 */
        column: string[]
        /** 表格展开列参数 并 排序 */
        expand: string[]

        /** 具体参数 */
        columns: columnsType
    }

    interface columnsType {
        [x: string]: columnsItemType
    }

    interface columnsItemType {
        /** 渲染表单组件类型 */
        type: 'input' | 'input-number' | 'select' | 'date-picker' | 'time-picker' | 'upload' | 'cascader' | 'radio-group'
        /** 标签名 */
        label: string
        /** 在展开行中隐藏 */
        hidden?: boolean
        /** 校验规则 */
        rules?: any
        /** 在编辑表单中所占的列数  */
        span?: number
        /** 表格列对齐方式 */
        align?: string
        /** 搜索表单默认值 */
        searchDefaultValue?: any
        /** 新增表单默认值 */
        addDefaultValue?: any
        /** 新增/编辑模式 显示状态 */
        disabled?: (Mode: string) => boolean
        /** 新增/编辑模式 显示状态 */
        visible?: (Mode: string) => boolean
        /** 是否为范围选择器 */
        range?: boolean
        /** 文件上传时图片预览地址 */
        fileList?: { name: string; url: string }[]
        /** 表单组件配置参数 */
        formProps?: any
        /** 表格配置参数 */
        tableProps?: any
        /** 表格列组件化插槽配置 */
        columnSlot?: any
    }
}
