import { ComponentConfig, DataStorage, CrudFunction } from './index'

export class CrudContext {
    // 单实例容器
    private static instance: CrudContext

    /** 组件配置 */
    componentConfig!: ComponentConfig
    /** 数据存储 */
    dataStorage!: DataStorage
    /** 方法 */
    crudFunction!: CrudFunction

    /** 存储注册组件时，传入的配置对象 */
    InitConfig!: Partial<ComponentConfig>

    /** 组件注册时，使用传入的 Config全局配置 进行初始化 */
    static InitGlobalConfig = (config: Partial<ComponentConfig>) => {
        // console.log('【1】、注册初始化\n', config, '\n初始化类容器')
        CrudContext.instance = new CrudContext(config)
    }

    // 私有的构造方法，无法使用new创建
    private constructor(config?: Partial<ComponentConfig>) {
        // console.log('【2】、保存注册的配置初始化全局类')
        if (config) this.InitConfig = config
    }

    /** 获取实例容器 */
    static getCrudCtx(): CrudContext {
        if (!CrudContext.instance) CrudContext.instance = new CrudContext()
        return CrudContext.instance
    }

    /**
     * 根据每个页面的Props进行初始化所有参数
     * @param props 组件Props
     */
    InitializationProps(props: ComponentConfig['PropsStore']) {
        // console.log('【5】、根据每个组件的 Prpos 值 进行组件的配置初始化')
        this.componentConfig = new ComponentConfig(this.InitConfig, props)
        this.dataStorage = new DataStorage(this.componentConfig)
    }

    /**
     * 初始化操作函数，根据props创建新的CrudFunction实例
     * @param extendedMethods 扩展CrudFunction的方法包含重写方法的对象
     */
    InitFunction(extendedMethods: Partial<CrudFunction>) {
        const newCrudFunction = new CrudFunction()
        if (!(Object.keys(extendedMethods).length === 0 && extendedMethods.constructor === Object)) {
            Object.assign(newCrudFunction, extendedMethods)
        }
        this.crudFunction = newCrudFunction
    }

    /** Debug方法 */
    CrudDebug(...params: any[]) {
        if (this.componentConfig.debug) {
            console.info('%c【DeBug】', 'color:red', ...params)
        }
    }
}
