import { UploadFile, UploadFiles, UploadRequestOptions } from 'element-plus'
import { reactive } from 'vue'
import { CrudContext } from '@/index'
import { nextTick } from 'vue'
import { CrudElType } from '@/vite-env'

let CrudCtx: CrudContext
nextTick(() => {
    CrudCtx = CrudContext.getCrudCtx()
})

export const source: CrudElType.SourceType = reactive({
    filter: ['input', 'numberInput', 'select', 'radio', 'date', 'time', 'deleteFlag'],
    editor: ['input', 'numberInput', 'select', 'radio', 'date', 'time', 'upload'],
    expand: [],
    column: ['input', 'numberInput', 'select', 'radio', 'date', 'time', 'upload'],

    columns: {
        input: {
            type: 'input',
            label: '输入框',
            // disabled: (Mode) => {
            //     return Mode === 'add'
            // },
            // visible: (Mode) => {
            //     // return Mode === 'add' || editModel.select === 1
            //     return Mode === 'add'
            // },
            rules: [{ required: true, message: '请输入', trigger: 'blur' }],
            formProps: {
                maxlength: 10,
                'show-word-limit': true,
                formatter: (value: any) => `$ ${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ','),
                parser: (value: string) => value.replace(/\$\s?|(,*)/g, '')
            }
        },
        numberInput: {
            type: 'input-number',
            label: '数字输入框',
            visible(Mode) {
                return Mode === 'add'
            },
            formProps: {
                precision: 3,
                step: 0.1,
                'controls-position': 'right'
            }
        },
        select: {
            type: 'select',
            label: '下拉选择器',
            formProps: {
                options: [
                    { value: 1, label: '选项1' },
                    { value: 2, label: '选项2' },
                    { value: 3, label: '选项3' }
                ]
                // multiple: true,
                // collapseTags: true,
                // collapseTagsTooltip: true,
                // filterable: true,
                // remote: true,
                // remoteMethod: (query: string) => {
                //     if (query) {
                //         //11111 console.log(query)
                //     }
                // }
            }
        },
        radio: {
            type: 'radio-group',
            label: '单选按钮组',
            span: 12,
            formProps: {
                // button: true,
                border: true,
                options: [
                    { value: 1, label: '选项1' },
                    { value: 2, label: '选项2' },
                    { value: 3, label: '选项3' }
                ]
            }
        },
        date: {
            type: 'date-picker',
            label: '日期选择',
            span: 12,
            formProps: {
                // disabledDate: (time: Date) => {
                //     return time.getTime() > Date.now()
                // }
                type: 'daterange',
                rangeSeparator: 'To',
                startPlaceholder: 'Start date',
                endPlaceholder: 'End date',
                format: 'YYYY/MM/DD',
                valueFormat: 'YYYY-MM-DD'
            }
        },
        time: {
            type: 'time-picker',
            label: '时间选择',
            formProps: {
                format: 'HH:mm:ss',
                valueFormat: 'HH:mm:ss'
            }
        },
        upload: {
            type: 'upload',
            label: '上传',
            formProps: {
                // drag: true, // 拖拽上传不要使用“picture-card”因为不好看
                listType: 'picture-card', // 多图上传，只需要添加这个就行了
                multiple: true,
                showFileList: true, // 默认是不显示上次列表的，如有需要需手动设置为True
                // URL: '', //当需要使用单图上传时，也就是只能显示一张图片。需要设置
                httpRequest: (files: UploadRequestOptions) => {
                    // 多文件上传时，使用push添加暂存就行了
                    CrudCtx.dataStorage.uploadFiles.value.push(files.file)

                    // 单图上传时，不使用默认的图片预览，使用el-image进行预览，需要设置图片的URL
                    // source.columns['upload'].formProps.URL = URL.createObjectURL(files.file)
                    // 单图上传时，不能用push，因为只能在uploadFiles中存放一个文件
                    // CrudCtx.dataStorage.uploadFiles.value[0] = files.file
                },
                // 因为使用httpRequest进行自定义上传，所以对于删除，预览等需要同步到uploadFiles对象中，要使用钩子进行操作。
                onRemove: (uploadFile: UploadFile, uploadFiles: UploadFiles) => {
                    // 可以：直接赋值处理后的数组
                    // CrudCtx.dataStorage.uploadFiles.value = uploadFiles

                    // 也可以：查找移除文件在数组中的下标
                    const id = CrudCtx.dataStorage.uploadFiles.value.findIndex((item) => item.uid === uploadFile.uid)
                    CrudCtx.dataStorage.uploadFiles.value.splice(id, 1)
                }
            }
        },
        deleteFlag: {
            type: 'radio-group',
            label: '状态',
            searchDefaultValue: 0,
            formProps: {
                button: true,
                options: [
                    { value: 0, label: '启用' },
                    { value: 1, label: '禁用' }
                ]
            }
        },
        operatorName: {
            type: 'input',
            label: ' 操作人账号'
        },
        createTime: {
            type: 'input',
            label: ' 创建时间'
        },
        updateTime: {
            type: 'input',
            label: ' 最后更新时间'
        },
        remark: {
            type: 'input',
            label: ' 备注'
        }
    }
})
